use gtk::{gio, glib, prelude::*, subclass::prelude::*};
use std::process::Command as Cmd;

mod imp {
    use std::cell::{OnceCell, RefCell};
    use super::*;

    #[derive(Debug, Default, glib::Properties, gtk::CompositeTemplate)]
    #[properties(wrapper_type = super::ImagePageWidget)]
    #[template(resource = "/ru/ximperlinux/Welcome/ui/image-page.ui")]
    pub struct ImagePageWidget {
        #[property(get, set = Self::set_resource_uri, construct_only)]
        pub(super) resource_uri: OnceCell<String>,
        #[property(get, set, construct_only)]
        pub(super) head: OnceCell<String>,
        #[property(get, set, construct)]
        pub(super) body: RefCell<Option<String>>,
        #[property(get, set, construct)]
        pub(super) button_label: RefCell<Option<String>>,
        #[property(get, set, construct)]
        pub(super) button_icon: RefCell<Option<String>>,
        #[property(get, set, construct)]
        pub(super) button_url: RefCell<Option<String>>,

        #[template_child]
        pub(super) picture: TemplateChild<gtk::Picture>,
        #[template_child]
        pub(super) container: TemplateChild<gtk::Box>,
        #[template_child]
        pub(super) action_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub(super) button_label_widget: TemplateChild<gtk::Label>,
        #[template_child]
        pub(super) button_icon_widget: TemplateChild<gtk::Image>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for ImagePageWidget {
        const NAME: &'static str = "ImagePageWidget";
        type ParentType = gtk::Widget;
        type Type = super::ImagePageWidget;

        fn class_init(klass: &mut Self::Class) {
            klass.set_layout_manager_type::<adw::ClampLayout>(); 
            klass.bind_template();
        }

        fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
            obj.init_template();
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for ImagePageWidget {
        fn constructed(&self) {
            if let Some(label) = self.button_label.borrow().clone() {
                self.button_label_widget.set_label(&label);
                self.action_button.set_visible(true);
            }

            if let Some(icon_name) = self.button_icon.borrow().as_ref() {
                let button_icon_widget = self.button_icon_widget.get();
                button_icon_widget.set_visible(true);
                button_icon_widget.set_icon_name(Some(&icon_name));
            }

            if let Some(url) = self.button_url.borrow().clone() {
                let button = self.action_button.get();
                
                button.connect_clicked(move |_| {
                    if url.starts_with("command://") {
                        let command = url.trim_start_matches("command://");
                        
                        let output = Cmd::new("sh")
                            .arg("-c")
                            .arg(command)
                            .output();
                        
                        match output {
                            Ok(output) => {
                                if output.status.success() {
                                    log::info!("Command executed successfully");
                                } else {
                                    log::error!("Command failed: {:?}", output.status);
                                }
                            }
                            
                            Err(e) => {
                                log::error!("Failed to execute command: {}", e);
                            }
                        }
                    } 
                    else {
                        let _ = gio::AppInfo::launch_default_for_uri(&url, None::<&gio::AppLaunchContext>);
                    }
                });
            }
        }

        fn dispose(&self) {
            self.container.unparent();
        }
    }

    impl WidgetImpl for ImagePageWidget {}

    impl ImagePageWidget {
        fn set_resource_uri(&self, resource_uri: &str) {
            self.resource_uri.set(resource_uri.to_owned()).unwrap();
            self.picture.set_resource(Some(resource_uri));
        }
    }
}

glib::wrapper! {
    pub struct ImagePageWidget(ObjectSubclass<imp::ImagePageWidget>)
        @extends gtk::Widget;
}
